;******************************************************************************
;    Filename:   DMX36	;    Date:  17-OCTOBER-2014;    File Version: 1.2;    Author:	Alan Hanson;    Company:	Hanson Electronics
;10-Oct-2014 1.1 had a bug where it would only save lower part of start address, intermittently missing the high byte of start address and therefore allowing an invalid address as valid
;25-Jun-2014 1.0 had a bug where it wouldn't save the start address
;******************************************************************************
	list		p=16f1823      ; list directive to define processor
	#include	 <P16F1823.INC>; processor specific variable definitions
;------------------------------------------------------------------------------
; CONFIGURATION WORD SETUP
; The 'CONFIG' directive is used to embed the configuration word within the .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration word settings.
;------------------------------------------------------------------------------    

;CHANGE AFTER DEBUGGING IS DONE
	__CONFIG 	_CONFIG1, _FOSC_HS & _WDTE_ON & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
	__CONFIG 	_CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_OFF & _BORV_LO & _LVP_OFF

; VARIABLE DEFINITIONS
; Available Data Memory divided into Bank 0-15.  Each Bank may contain  Special Function Registers, General Purpose Registers, and Access RAM 
;------------------------------------------------------------------------------

;*******************RAM register definitions**********************
	CBLOCK	0x020
	WREG_TEMP	;variable used for context saving	; These 1ST 3 variables are only needed if low priority interrupts are used.  
	STATUS_TEMP	;variable used for context saving
	BSR_TEMP	;variable used for context saving
	TEMP	;SHORT TERM TEMP REGISTER
	TEMP2	;SHORT TERM TEMP REGISTER2
	TEMP3	;SHORT TERM TEMP REGISTER3
	TEMP4	;SHORT TERM TEMP REGISTER4
	RUNMODE	;7-NORMAL RUNNING, 6-TEST MODE, 4-NO DMX, 3-ADDRESS PGM MODE, 2-ADDRESS RCVD
	T_MODE	;TEST MODE REGISTER. NUMBER IN HERE CORRSPONDS TO CURRENT TEST MODE MODE. SEE SECTION OF CODE FOR DETAILS
	T_TIME	;A 0.1S INCREMENTED TIMER TO KEEP TRACK OF HOW LONG IN EACH OF THE TEST MODES
	DmxPks	;USED TO COUNT THE NUMBER OF INCOMING PACKETS. ONLY COUNTS TO 8. ANY LESS WON'T GENERATE THE PART PACKETS ERROR
	LEDCNTR	;COUNTER USED FOR THE FLASHING OF THE STATUS LEDS AND FOR GENERAL TIMING. CLOCKED AT ABOUT 10Hz BY TIMER 0
	Cnt_H 	;16-bit counter  HIGH (USED TO COUNT TO 512 FOR THE DMX512 512 BITS OF DATA)
	Cnt_L	;16-bit counter  LOW  (USED TO COUNT TO 512 FOR THE DMX512 512 BITS OF DATA)
	DmxCount	;Counter to check that all 36 channels are recieved
	DmxCntDn	;Countdown timer used for checking for dmx packets. loaded with 30 each packet and counts down every 0.1s
	MODE	;COPY OF MODE SWITCH AS WELL AS TEST MODE SWITCH FROM ADDRESS SWITCH (BIT 5 COMES FROM TEST MODE SETTING ON ADDRESS SWITCH)
	DMXLOW	;USED TO STORE DMX START ADDRESS FROM STREAMED DATA (LOWER 8 BITS)
	DMXHIGH	;USED TO STORE DMX START ADDRESS FROM STREAMED DATA (UPPER 1 BITS)
	TIMER1	;USED BECAUSE TMR IS ONLY 8 BIT
	Add_ByteH	;1st byte received when in address setting mode
	Add_ByteL	;2nd byte received when in address setting mode

	PWM1 	;brightness control LED1
	PWM2 	;brightness control LED2
	PWM3 	;brightness control LED3
	PWM4 	;brightness control LED4
	PWM5 	;brightness control LED5
	PWM6 	;brightness control LED6
	PWM7 	;brightness control LED7
	PWM8 	;brightness control LED8
	PWM9 	;brightness control LED9
	PWM10 	;brightness control LED10
	PWM11 	;brightness control LED11
	PWM12 	;brightness control LED12
	PWM13 	;brightness control LED13
	PWM14 	;brightness control LED14
	PWM15 	;brightness control LED15	
	PWM16 	;brightness control LED16
	PWM17	;brightness control LED17
	PWM18	;brightness control LED18
	PWM19	;brightness control LED19
	PWM20	;brightness control LED20
	PWM21	;brightness control LED21
	PWM22	;brightness control LED22
	PWM23	;brightness control LED23
	PWM24	;brightness control LED24
	PWM25 	;brightness control LED25
	PWM26 	;brightness control LED26
	PWM27 	;brightness control LED27
	PWM28 	;brightness control LED28
	PWM29 	;brightness control LED29
	PWM30 	;brightness control LED30
	PWM31 	;brightness control LED31
	PWM32 	;brightness control LED32
	PWM33 	;brightness control LED33
	PWM34 	;brightness control LED34
	PWM35 	;brightness control LED35
	PWM36 	;brightness control LED36

;THERE MAY HAVE TO BE 2 BANKS OF RAM USED	
	RxBuffer: 	.1	;THIS 1 BYTE IS ALLOCATED TO STORE THE START ADDRESS THAT IS SENT AS PART OF DMX STREAM
	RxBuffer1: 	.36 	;36 bytes buffer allocation for recieved data
    ENDC

;------------------------------------------------------------------------------
; EEPROM INITIALIZATION   The 16F1823 has 256 bytes of non-volatile EEPROM, starting at address 0xF000
DATAEE    ORG  0xF000
    DE    0,0,0,1," Version_1.2"  ; 1ST 2 16 BIT BYTES ARE THE DEFAULT START ADDRESS THEN A SPACE. THEN software version after that

	#DEFINE	MODE_LED	LATC,0	;Output for mode led
	#DEFINE	DATA_LED	LATA,2	;Output for data led
	#DEFINE	TEST_SW	PORTC,3	;Test switch dipswitch input
	#DEFINE	PGM_SW	PORTC,4	;Program mode dipswitch input
	#DEFINE	SER_CK	LATC,1	;2803 SERIALCLOCK
	#DEFINE	SER_DA	LATC,2	;2803 SERIAL DATA
;------------------------------------------------------------------------------
; RESET VECTOR
    ORG     	0x0000            ; processor reset vector
    GOTO    	MAIN             ; When using debug header, first inst.  ; may be passed over by ICD2.  
;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
    ORG      	0x0004
	nop
;------------------------------------------------------------------------------
; USER INTERRUPT SERVICE ROUTINE GOES HERE
;------------------------------------------------------------------------------
; Note the 16F1933 family automatically handles context restoration for  W, STATUS, BSR, FSR, 
;and PCLATH where previous templates for 16F families required manual restoration

    RETFIE                    ; return from interrupt
;******************************************************************************
MAIN:	;Start of main program
	clrwdt
	CALL	CHIPINIT	;SETUP ALL THE PORTS ETC
	CALL	REGINIT	;SETUP ALL THE REGISTERS ASSOCIATED WITH THE 2803 CHIP
	CALL	WRITE_2803	;COPY FROM THE PWM REGISTERS TO THE 2803's

MAINLOOP:	CLRWDT	;THE MAIN PROGRAM CYCLES FROM HERE ON

	CLRF	BSR

MAIN2:	NOP
	BANKSEL	PIR1
	btfsc	PIR1,RCIF 	; wait until a byte is correctly received
	CALL	GetData
	BANKSEL	INTCON	
	BTFSS	INTCON,2	;TMR0IF	;CHECK TO SEE IF TIMER 0 HAS ROLLED OVER. approx 10Hz
	GOTO	MAINLOOP
   ;the 10Hz timer has expired
	BANKSEL	INTCON
	BCF	INTCON,2;TMR0IF	;CLEAR THE TIMER 0 INTERRUPT FLAG
	INCFSZ	TIMER1,F
	GOTO	MAINLOOP
	MOVLW	D'248'
	MOVWF	TIMER1
;HANDLE THE NO DMX TIMER
	MOVF	DmxCntDn,W
	BZ	MAIN2A		;IF NUMBER IS ZERO THEN SKIP TO NEXT SECTION	
	DECF	DmxCntDn,F		;DECREMENT THE CHECK TIMER. 30 IS LOADED EACH PACKET AND COUNTS DOWN EACH 0.1s
	BZ	MAIN2A		;IF NUMBER IS ZERO THEN SKIP TO NEXT SECTION
	GOTO	MAIN3
MAIN2A	;DmxCntDn is zero
	BSF	RUNMODE,4		;SET THE NO DMX BIT
	CLRF	DmxCntDn		;BRING IT BACK TO ZERO
	BTFSC	RUNMODE,3		;if in address setting mode
	CALL	AllOff		;then ensure all the leds are turned off
	BTFSS	RUNMODE,6
	CALL	AllOff
	BTFSS	RUNMODE,6	
	CALL	WRITE_DMX
	BTFSS	RUNMODE,6
	CALL	WRITE_2803
	GOTO	MAIN3

;CHECK IF TEST SWITCH OR PROGRAM SWITCH IS ON
MAIN3	BTFSS	TEST_SW	;CHECK TEST SWITCH
	BSF	RUNMODE,6	;TEST MODE
	BTFSC	TEST_SW
	BCF	RUNMODE,6
	BTFSC	RUNMODE,6
	GOTO	MAIN4
	BTFSC	PGM_SW	;CHECK PROGRAM SWITCH IS OFF
	BSF	RUNMODE,7	;NORMAL RUNNING MODE
	BTFSS	PGM_SW
	BCF	RUNMODE,7
	BTFSS	PGM_SW	;CHECK PROGRAM SWITCH IS ON
	BSF	RUNMODE,3	;ADDRESS SETTING MODE
	BTFSC	PGM_SW	;CHECK PROGRAM SWITCH 
	BCF	RUNMODE,3	;TURN OFF PROGRAMMING MODE BIT
	BTFSC	PGM_SW	;CHECK PROGRAM SWITCH 
	BCF	RUNMODE,2	;TURN OFF PROGRAMMING MODE ADDRESS RCVD BIT

;RUNMODE	;7-NORMAL RUNNING, 6-TEST MODE, 4-NO DMX,3-ADDRESS PGM MODE,2-ADDRESS RCVD
;DO ALL THE CHECKS TO WORK OUT WHAT MODE THE LEDS NEED TO BE FLASHED IN
MAIN4	INCF	LEDCNTR,F	;INCREMENT THE COUNTER USED FOR FLASHING LEDS
	BTFSC	RUNMODE,6	;CHECK IF IN TEST MODE
	CALL	TESTMODE	;UPDATE THE LED DATA IN MEMORY
	BTFSC	RUNMODE,6	;CHECK IF IN TEST MODE
	CALL	WRITE_DMX	;COPY FROM DMX RECEIVE REGISTERS TO THE PWM REGISTERS
	BTFSC	RUNMODE,6	;CHECK IF IN TEST MODE
	CALL	WRITE_2803	;COPY FROM THE PWM REGISTERS TO THE 2803's
	BTFSC	RUNMODE,6	;CHECK IF IN TEST MODE
	GOTO	FLSHLD_2
	BTFSC	RUNMODE,3	;CHECK IF IT IS IN ADDRESS SETTING MODE
	GOTO	FLSHLD_1
	BTFSC	RUNMODE,7	;CHECK TO SEE IF IN NORMAL RUNNING MODE
	GOTO	FLSHLD_3

ENDLEDS	GOTO	MAINLOOP

;MODE AND DATA LED FLASHING
;RUNMODE	;7-NORMAL RUNNING, 6-TEST MODE, 4-NO DMX,3-ADDRESS PGM MODE,2-ADDRESS RCVD
FLSHLD_1	NOP
	BANKSEL	PORTA	;ADDRESS MODE, GREEN (DATA) IS OFF UNTIL RX 1. BLUE(MODE) FLASHES AT ABOUT 10Hz
	BCF	DATA_LED
	BTFSC	RUNMODE,2	;IF A VALID ADDRESS HAS BEEN RECEIVED THEN
	BSF	DATA_LED	;TURN ON THE DATA LED
	BTFSS	LEDCNTR,0	;FLASH THE MODE LED
	BSF	MODE_LED	;AT LEDCNTR RATE
	BTFSC	LEDCNTR,0
	BCF	MODE_LED
	GOTO	ENDLEDS

FLSHLD_2	NOP
	BANKSEL	PORTA	;TEST MODE , GREEN (DATA) IS OFF. BLUE(MODE) FLASHES AT ABOUT 1Hz
	BCF	DATA_LED  	
	BTFSS	LEDCNTR,3
	BSF	MODE_LED
	BTFSC	LEDCNTR,3
	BCF	MODE_LED
	GOTO	ENDLEDS

FLSHLD_3	NOP
	BANKSEL	PORTA 	;NORMAL RUNNING MODE. FLASH THE DATA LED DEPENDANT ON DMX DATA. MODE LED ON SOLID
	BSF	MODE_LED	
	BTFSC	RUNMODE,4	;CHECK FOR NO DMX
	GOTO	FLSHLD3A		
	BTFSS	LEDCNTR,0	;MUST BE GETTING COMPLETE PACKETS SO FLASH AT 10Hz
	BSF	DATA_LED
	BTFSC	LEDCNTR,0
	BCF	DATA_LED
	GOTO	ENDLEDS
FLSHLD3A	BCF	DATA_LED	;NO DMX SO TURN OFF DATA LED
	GOTO	ENDLEDS

;EEPROM WRITE ROUTINE. WRITES THE 2 BYTES OF START ADDRESS
WRITE_EE	NOP
	BANKSEL 	EEADRL 		;
	MOVLW 	D'1'	 	;1ST BYTE IS TO BE WRITTEN
	MOVWF 	EEADRL 		;Data Memory Address to write
	CLRF	EEADRH
	BANKSEL	Add_ByteH	
	MOVF	Add_ByteH,W		;GRAB THE 1ST BYTE OF START ADDRESS
	BANKSEL	EEDATL
	MOVWF 	EEDATL 		;Data Memory Value to write
	BCF 	EECON1, CFGS 	;Deselect Configuration space
	BCF 	EECON1, EEPGD 	;Point to DATA memory
	BSF 	EECON1, WREN 	;Enable writes
	MOVLW 	55h 		;REQUIRED PROCESS NEXT COUPLE STEPS
	MOVWF 	EECON2 		;Write 55h
	MOVLW 	0AAh 		;
	MOVWF 	EECON2 		;Write AAh
	BSF 	EECON1, WR 	;Set WR bit to begin write
	BTFSC 	EECON1, WR 	;Wait
	GOTO 	$-2 		;Done
	BANKSEL 	EEADRL 		;
	MOVLW 	D'3' 		;2ND BYTE IS TO BE WRITTEN
	MOVWF 	EEADRL		;
	BANKSEL	Add_ByteL		;
	MOVF	Add_ByteL,W		;GRAB THE 2ND BYTE OF START ADDRESS
	BANKSEL	EEDATL		;CHANGE TO CORRECT BANK
	MOVWF 	EEDATL 		;Data Memory Value to write
	MOVLW 	55h 		;REQUIRED PROCESS NEXT COUPLE STEPS
	MOVWF 	EECON2 		;Write 55h
	MOVLW 	0AAh ;
	MOVWF 	EECON2 		;Write AAh
	BSF 	EECON1, WR 	;Set WR bit to begin write
	BCF 	EECON1, WREN 	;Disable writes
	BTFSC 	EECON1, WR 	;Wait for write to complete
	GOTO 	$-2 		;Done
	BANKSEL	ADD_BYTEL
	MOVF	ADD_BYTEL,W
	BANKSEL	DMXLOW
	MOVWF	DMXLOW
	BANKSEL	ADD_BYTEH
	MOVF	ADD_BYTEH,W
	BANKSEL	DMXHIGH
	MOVWF	DMXHIGH
	RETURN

;***********************to receive DMX-512 data and store it ; into 36 byte receive buffer. ONLY USES THE REQUIRED CHANNELS IF NOT ALL 512 SENT  **********
DmxClear	BCF	RCSTA,FERR
	BANKSEL	RCREG
	MOVF	RCREG,W	;LOAD THE BUFFER TO CLEAR THE ERROR
	BANKSEL	Cnt_H
	MOVF	Cnt_H,W
	BNZ	CLEAR2
	DECF	Cnt_L,W
	BNZ	CLEAR2
	RETURN

CLEAR2	CLRF 	Cnt_L	; init receive counter and buffer pointer
	CLRF 	Cnt_H
	CLRF	DmxCount	;Counter to check that all 36 channels are recieved
	RETURN

GetData	nop
;moved from main loop
	BANKSEL	RCSTA
	BTFSS	RCSTA,OERR
	BRA	Data2
	BCF	RCSTA,CREN
	BSF	RCSTA,CREN
	BANKSEL	CNT_L
	CLRF	CNT_L
	CLRF	CNT_H
;moved from main loop
Data2	NOP
	BANKSEL	PIR1
	BCF	PIR1,RCIF		;CLEAR THE INTERRUPT FLAG
	BANKSEL	RCSTA
	BTFSC 	RCSTA,FERR 	; if a new framing error is detected (error or short frame)
	GOTO	DmxClear

	BANKSEL	RCREG
	MOVF	RCREG,W
	BANKSEL	RUNMODE
	BTFSC	RUNMODE,6	;CHECK IF IN TEST MODE
	RETURN		;IF IT IS THEN DMX PROCESSSING DOESN'T NEED TO BE DONE
	BANKSEL	STATUS
	CLRF	STATUS
	BANKSEL	Cnt_L
	INCF 	Cnt_L,F 	; increment 16-bit counter
	BTFSS 	STATUS,Z	;STATUS,
	GOTO	ADD_MODE	;WAS StartBt
	INCF 	Cnt_H,F

;Check if in address setting mode
ADD_MODE	BTFSS	RUNMODE,3	
	GOTO	StartBt
	BTFSC	RUNMODE,2	;CHECK IF  ADDRESS HAS ALREADY BEEN SAVED
	RETURN
	MOVF	Cnt_H,F
	BTFSS	STATUS,Z
	RETURN
	MOVLW	D'2'
	XORWF	Cnt_L,W
	BTFSC	STATUS,Z
	GOTO	SAVE_LOW
	MOVLW	D'3'
	XORWF	Cnt_L,W
	BTFSC	STATUS,Z
	GOTO	SAVE_HIGH
	RETURN

SAVE_LOW	NOP
	clrf	TMR0
	BANKSEL	RCREG	;grab the copy of the data in RCREG (dmx byte 1)
	MOVF	RCREG,W
	BANKSEL	Add_ByteL
	MOVWF	Add_ByteL	;save it into Add_ByteL
	RETURN
SAVE_HIGH	NOP
	BANKSEL	RCREG	;grab a copy of the data in RCREG (dmx byte 2)
	MOVF	RCREG,W
	BANKSEL	Add_ByteH
	MOVWF	Add_ByteH	;save it into Add_ByteH

;check to see if the address is within the usuable address range. usable range is 1 to 477 (256 +221)
;check for 2 zeroes
	MOVF	Add_Byteh,W	;CHECK IF HIGH BYTE WAS ZERO
	BTFSS	STATUS,Z
	GOTO	NotZero
	MOVF	Add_ByteL,W	;CHECK IF LOW BYTE WAS ZERO
	BTFSC	STATUS,Z
	RETURN	;If both addresses were zero then don't save the data
	GOTO	ValidAddr	;If low >0 and high was 0 then address must be valid

NotZero	;checks if add high is > 1 and if it's 1 then if the add low is higher than 221
	decfsz	Add_Byteh,W	;CHECK IF HIGH BYTE WAS ZERO
	RETURN
	
IsOne	MOVLW	B'00100010'	;THIS IS 255 minus 221 WHICH IS THE MAXIMUM THAT LOWER 8 BITS CAN BE
	ADDWF	Add_ByteL,W
	BTFSC	STATUS,C
	GOTO	PastAddr

ValidAddr	NOP
	BSF	RUNMODE,2	;SET THE RECIEVED ADDRESS BIT
	CALL	WRITE_EE	;SAVE THE 2 BYTES INTO EEPROM
	MOVF	Add_ByteL,W
	MOVWF	DMXLOW	;SAVE TO THE LOW START ADDRESS
	MOVF	Add_ByteH,W
	MOVWF	DMXHIGH	;SAVE TO THE HIGH START ADDRESS
	RETURN
	
StartBt	;CHECK TO SEE IF 36 BYTES HAVE BEEN CLOCKED IN
	MOVF	DmxCount,W	;Counter to check that all 36 channels are recieved
	XORLW	D'37'
	BTFSC	STATUS,Z	;CHECK TO SEE IF ALL 36 ARE CLOCKED IN
	GOTO	GrtEq_end	;IF THEY ARE THEN SKIP TO THE END

ADDCHK	;CHECK TO SEE IF CURRENT BYTE IS ABOVE START ADDRESS
	MOVF	DmxCount,w	;Check to see if counter is above zero. If it is then start address must have been reached
	BNZ	GrtrEq
	MOVF	DMXHIGH,w
	XORWF	Cnt_H,W
	BTFSS	STATUS,Z
	GOTO	MOVE3
	GOTO	ChkLow

ChkLow	MOVF	DMXLOW,w
	SUBWF	Cnt_L,W
	BTFSC	STATUS,Z
	GOTO	GrtrEq	;ADDRESS WAS THE SAME
	BTFSS	STATUS,C	;***************** was BTFSS STATUS,DC **********************88
	GOTO	MOVE3	;ADDRESS WAS TOO LOW
	GOTO	GrtrEq

GrtrEq	;CHANNEL IS GREATER OR EQUAL START CHANNEL SO INCREMENT THE DMXCOUNT AND SEE IF IT'S REACHED 36
	INCF	DmxCount,F	;Counter to check that all 36 channels are recieved
	CLRF	FSR0H
	MOVLW	RxBuffer
	MOVWF	FSR0L
	DECF	FSR0L,F
	MOVF	DmxCount,W
	ADDWF	FSR0L,F
	BANKSEL	RCREG
	MOVF	RCREG,W
	CLRF	BSR	;GO BANK TO BANK 0
	MOVWF	INDF0	;SAVE THE RCV BUFFER INTO THE DMX RCV REGISTER
	MOVLW	D'30'	;LOAD UP A 3s VALUE INTO THE CHECK TIMER
	MOVWF	DmxCntDn

PastAddr	MOVF	DmxCount,W	;Counter to check that all 36 channels are recieved
	XORLW	D'37'
	BTFSC	STATUS,Z	;CHECK TO SEE IF ALL 36 ARE CLOCKED IN
	GOTO	MOVE2B	;IF THEY ARE NOW THEN WRITE THE DATA OUT

    ;RUNMODE	;7-NORMAL RUNNING, 6-TEST MODE,5-OTHER MODE, 4-NO DMX,3-NO DMX FADE DOWN, 2-ONLY PART PACKETS,1-NOT USED, 0-ADDRESS INVALID
GrtEq_end	GOTO	MOVE3

MOVE2B	BTFSS	RUNMODE,3	;RETURN
	BCF	RUNMODE,4	;clear the no dmx bit
	CALL	WRITE_DMX	;copy from rcv buffers to pwm registers
	CALL	WRITE_2803	;copy from pwm registers to outputs
MOVE3	RETURN

;********************************************************* Setup Serial port and buffers ******************************************************
SetupSerial	NOP
	BANKSEL	Cnt_L	;Clear the receive buffer;************************************
	CLRF	Cnt_L
	CLRF	Cnt_H
	MOVLW	RxBuffer
	MOVWF	FSR0L
	CLRF	FSR0H
	MOVLW	D'36'
	MOVWF	TEMP
ClrBuffLoop	CLRF 	INDF0 	; clear INDF register then increment pointer
	INCF	FSR0L,F
	DECF	TEMP,F
	BTFSS 	STATUS,Z
	GOTO 	ClrBuffLoop

; Setup EUSART
	BANKSEL	TRISC
	BSF 	TRISC,5 	; allow the EUSART RX to control pin RC5
	MOVLW 	0x04 	; Disable transmission
	BANKSEL	TXSTA
	MOVWF 	TXSTA 	; enable transmission and CLEAR high baud rate
	MOVLW 	0x90
	MOVWF 	RCSTA 	; enable serial port and reception
	BSF 	BAUDCON,BRG16 ; Enable UART for 16-bit Asyn operation
	BSF	BAUDCON,SCKP  ;INVERT THE DATA TO SUIT SP485 RATHER THAN MAX1483
	CLRF 	SPBRGH
	MOVLW 	.19 	; Baud rate is 250KHz for 16MHz Osc. freq.
	MOVWF 	SPBRG
	RETURN		;End of DMX RECEIVAL program
;********************************** END OF DMX RECIEVE ROUTINE ******************************************

;********************************** COPY FROM DMX RECEIVE REGISTERS TO THE PWM REGISTERS ******************************************
WRITE_DMX		
    ;DATA IS ORIGINALLY STORED IN 36 REGISTERS FROM RxBuffer1. Copy it to PWM1 etc	
  ;data needs mapping according to 
;rx1-9 	tx27-19
;rx18-10	tx18-10
;rx19-27	tx36-28
;rx28-36	tx9-1
	
	CALL	SAVE_FSR	;SAVE WHAT WAS IN THE FSR IN CASE IT WAS IN USE
	CLRF	BSR
	MOVLW	RxBuffer1
	MOVWF	FSR0L
	CLRF	FSR0H	;COPY THE START ADDRESS OF THE RX BUFFER (PLUS 27) INTO FSR0
	MOVLW	PWM1+D'17'
	MOVWF	FSR1L	;COPY THE START ADDRESS OF PWM1 INTO THE FSR1
	CLRF	FSR1H
    ;COPY THE PWM DATA ACROSS
	MOVLW	D'36'	;LOAD 36 INTO
	MOVWF	TEMP3	;TEMP3 READ TO COPY THE 36 BYTES ACROSS
WRT_LP1	MOVF	INDF0,W	;LOAD THE DATA FROM RxBuffer
	MOVWF	INDF1	;COPY IT TO PWM1 etc
	INCF	FSR0L,F
	DECF	FSR1L,F	;DECREMENT THE destination REGISTER AS THEY ARE MAPPED IN REVERSE ORDER

	MOVLW	D'10'
	XORWF	TEMP3,W
	BTFSC	STATUS,Z	;IF 9 BYTES remaining IT IS TIME TO CHANGE
	GOTO	WRITE_9
	MOVLW	D'19'
	XORWF	TEMP3,W
	BTFSC	STATUS,Z	;IF 18 BYTES remaining IT IS TIME TO CHANGE
	GOTO	WRITE_18
	MOVLW	D'28'
	XORWF	TEMP3,W
	BTFSC	STATUS,Z	;IF 27 BYTES remaining IT IS TIME TO CHANGE
	GOTO	WRITE_27
	GOTO	WRITE_END

WRITE_9	MOVLW	PWM1+D'8'	;9 bytes remaining so
	MOVWF	FSR1L	;COPY THE START ADDRESS OF PWM1 INTO THE FSR1
	GOTO	WRITE_END
WRITE_18	MOVLW	PWM1+D'26'	;18 bytes remaining so
	MOVWF	FSR1L	;COPY THE START ADDRESS OF PWM1 INTO THE FSR1
	GOTO	WRITE_END
WRITE_27	MOVLW	PWM1+D'35'	;27 bytes remaining so
	MOVWF	FSR1L	;COPY THE START ADDRESS OF PWM1 INTO THE FSR1

WRITE_END	DECF	TEMP3,F
	BNZ	WRT_LP1

	CALL	RESTORE_FSR
	RETURN
;****************************END OF COPY FROM DMX RECEIVE REGISTERS TO THE PWM REGISTERS ******************************************

;************************************************************* COPY FROM THE PWM REGISTERS TO THE 2803's  *********************************
WRITE_2803	    ;*******************Begin Main WRITE Program****************************
	CLRF	BSR
	MOVLW   	PWM1       ;COPY THE START ADDRESS OF PWM1 
	MOVWF   	FSR0L	  ;INTO THE FSR0 
	CLRF    	FSR0H	  ;
	MOVLW   	D'36'	;no_of_channels  ;
	MOVWF   	TEMP3	;USED TO KEEP TRACK OF HOW MANY BYTES ARE SENT
	MOVLW	D'8'
	MOVWF	TEMP2	;USED TO KEEP TRACK OF NUMBER OF BITS THAT HAVE BEEN SENT
bitbang:	nop
	BANKSEL	LATC
	BCF	SER_CK	;SET THE CLOCK LOW JUST IN CASE IT HADN'T BEEN SET LOW
	CLRF	BSR
	movf    	INDF0,w	;GRAB A COPY OF THE RELEVANT PWM
;	movlw	b'11111110'	;test code
	MOVWF	TEMP4	;COPY IT TO TEMP4
	;clock is at c1, data is at c2

BITBANG2	CLRWDT
	BANKSEL	LATC
	MOVF	LATC,W
	CLRF	BSR
	MOVWF	TEMP	;USED FOR STORING A COPY OF PORTC
	BSF	TEMP,2	;TURN ON BIT THEN CHECK IF IT SHOULD BE ON
	BTFSS	TEMP4,7	;IF THE BIT CURRENTLY IN 7 IS
	BCF	TEMP,2	;A ZERO THEN CLEAR THAT BIT
	MOVF	TEMP,W
	BANKSEL	LATC
	MOVWF	LATC	;COPY THE DATA TO THE PORT
	NOP
	BSF	SER_CK	;CLOCK THE DATA OUT
	NOP
	BCF	SER_CK	;CLEAR THE CLOCK BIT
	CLRF	BSR
	RLF	TEMP4,F
	DECFSZ	TEMP2,F
	GOTO	BITBANG2
	MOVLW	D'8'
	MOVWF	TEMP2	;load the 8 bits that need to get clocked out each time
	INCF   	FSR0L,F	;increment the offset register so it grabs the next data
	decfsz  	TEMP3,f       ;	HAS THE LAST BYTE BEEN WRITTEN OUT?
	goto    	bitbang	;

	BANKSEL	LATC
	BCF	SER_CK	;SET THE CLK OUTPUT LOW
	BCF	SER_DA	;SET THE DTA OUTPUT LOW TOO
	CLRF	BSR
        	call    	delay_1mS	;
	RETURN

delay_1mS	;1 ms delay to save stuff in 2803 and get ready to send next lot of packets
	CLRF	TEMP
	MOVLW	D'247'
	MOVWF	TEMP2

DELAY_PT1	INCFSZ	TEMP,F
	GOTO	DELAY_PT1
DELAY_PT2	CLRWDT
	INCFSZ	TEMP2,F
	GOTO	DELAY_PT1			
	RETURN

;******************************************************************** save and restore fsr0 in case it was in use ********************************
SAVE_FSR	;SAVE THE FSR IN CASE IT WAS IN USE
	MOVF	FSR0L,W	;GRAB THE FSR REGISTERS IN CASE
	MOVWF	TEMP	;THEY WERE IN USE.
	MOVF	FSR0H,W	;SAVE THEM AGAIN
	MOVWF	TEMP2	;LATER
	RETURN
RESTORE_FSR	;RESTORE THE FSR IN CASE IT WAS IN USE
	MOVF	TEMP,W	;SAVE THE FSR
	MOVWF	FSR0L
	MOVF	TEMP2,W
	MOVWF	FSR0H
	RETURN
;******************************************** SETS UP ALL THE TRIS, INTERUPTS, ETC WHEN THE CHIP IS 1ST POWERED UP ***********************************
CHIPINIT:	NOP
	BANKSEL	INTCON
	CLRF	INTCON		;MAKE SURE ALL INTERRUPTS ARE OFF UNTIL I'M READY FOR THEM
	CLRF	PIR1		;CLEAR PERIPHERAL INTERRUPT FLAG REGISTER 1
	CLRF	PIR2		;CLEAR PERIPHERAL INTERRUPT FLAG REGISTER 2
	BANKSEL	WDTCON
	MOVLW	b'00000111'		;Set WDT Prescaler  128mS interval
	MOVWF	WDTCON
	MOVLW	b'11111011'		;PORT A CONFIGURED AS ALL INPUTS EXCEPT A2
	MOVWF	TRISA		;
	MOVLW	B'11111000'		;PORT B ALL INPUTS EXCEPT C0,C1,C2
	MOVWF	TRISC
	BANKSEL	LATA		;CHANGE TO BANK THAT LATCHES ARE IN FOR PORT A
	CLRF	LATA		; All PORTA off
	CLRF	LATC		; All PORTC off
	BANKSEL	APFCON
	CLRF	APFCON	
	BANKSEL	WPUA
	MOVLW	B'00111111'		;PULLUPS ON ALL INPUTS OF PORT A
	MOVWF	WPUA	
	MOVLW	B'00111111'		;PULLUPS ON ALL INPUTS OF PORT C	
	MOVWF	WPUC
	BANKSEL	ANSELA
	CLRF	ANSELA
	CLRF	ANSELC
	CALL 	SetupSerial 		;Setup Serial port and buffers

	BANKSEL	OPTION_REG
	MOVLW	B'00000111'
	MOVWF	OPTION_REG	;WEAK PULLUPS ON. SET THE PRESCALER FOR TIMER 0 AS 256
	CLRF	CPSCON0		;Timer0 clock source is controlled by the core/Timer0 module and is FOSC/4
	BANKSEL	TMR0
	CLRF	TMR0		;CLEAR THE TIMER 0 LOW BYTE
	CLRF	BSR
	CLRF	DmxCount		;Counter to check that all 36 channels are recieved
	CLRF	DmxCntDn		;dmx countdown timer. loaded with 30 each packet and counts down each 0.1s
	CLRF	T_MODE
	CLRF	T_TIME
	CLRF	MODE
	MOVLW	D'250'
	MOVWF	TIMER1
	CLRF	RUNMODE
	BSF	RUNMODE,4		;SET THE NO DMX BIT

	BANKSEL 	EEADRL 	;	GRAB THE 2 BYTES OF THE START ADDRESS
	MOVLW 	D'3' 	;	;Address to read
	MOVWF 	EEADRL 		;Data Memory
	BCF 	EECON1, CFGS 	;Deselect Config space
	BCF 	EECON1, EEPGD	;Point to DATA memory
	BSF 	EECON1, RD 		;EE Read
	MOVF 	EEDATL, W 		;W = EEDATL
	CLRF	BSR
	MOVWF	DMXLOW
	BANKSEL 	EEADRL 	;
	MOVLW 	D'1' 	;	;Address to read
	MOVWF 	EEADRL 		;Data Memory
	BCF 	EECON1, CFGS 	;Deselect Config space
	BCF 	EECON1, EEPGD	;Point to DATA memory
	BSF 	EECON1, RD 		;EE Read
	MOVF 	EEDATL, W 		;W = EEDATL
	CLRF	BSR
	MOVWF	DMXHIGH

	RETURN
;******************************************* END OF CHIP INITIALISATION*****************************************************************************************
;************************CLEARS/INITIALISES ALL THE REGISTERS/MEMORY THAT ARE USED FOR 2803's**************************************************
REGINIT	;FIRST SECTION SETS UP ALL THE GROUP A DATA	
  	MOVLW	D'36'
	MOVWF	TEMP
	MOVLW	PWM1
	MOVWF	FSR0L	;LOAD THE INDIRECT REGISTER LOWER BYTE WITH THE START ADDRESS OF THE LED PWM REGISTERS
	CLRF	FSR0H	;CLEAR THE INDIRECT REGISTER HIGH BYTE	
	
CLR_PWM	;CLEAR ALL OF THE PWM REGSISTERS
	CLRF	FSR0	;CLEAR THE PWM DATA THAT MATCHES THE SELF INCREMENTING OFFSET REGISTER
	INCF	FSR0L,F	
	MOVF	TEMP,F	;Counter to check that all 36 channels are recieved
	DECFSZ	TEMP,F	;skip if all 36 registers are cleared
	GOTO	CLR_PWM
	RETURN
;************************************************************************************* END OF REGISTRY INITIALISATION ********************
;************************************************************************************ TEST MODE **********************************************
;0	START OF TEST MODE. CLEAR T_TIME AND ALL PWM REGISTERS
;1	SINGLE	LED1 THEN 2 THEN 3 ----> 36 . 	@ 0.1S. TOTAL 6S
;2	SINGLE R	LED1 THEN 4 THEN 7 ETC		@ 0.2S. TOTAL 4S
;3	SINGLE G	LED2 THEN 5 THEN 8 ETC		@ 0.2S. TOTAL 4S
;4	SINGLE B	LED3 THEN 6 THEN 9 ETC		@ 0.2S. TOTAL 4S
;5	ALL RED	LED 1 + LED 4 + LED 7 ETC		@ 1S.    TOTAL 1S
;6	ALL GREEN	LED 2 + LED 5 + LED 8 ETC		@ 1S.    TOTAL 1S
;7	ALL BLUE	LED 3 + LED 6 + LED 9 ETC		@ 1S.    TOTAL 1S
;8	AQUA (G+B) 	LED 2 +LED 3 + LED 5 + LED 6 ETC	@ 1S.    TOTAL 1S
;9	YELLOW (R+G)	LED 1 + LED 2 + LED 4 + LED 5 ETC	@ 1S.    TOTAL 1S
;10	PURPLE (R+B)	LED 1 + LED 3 + LED 4 + LED 6 ETC	@ 1S.    TOTAL 1S
;11	WHITE	ALL LEDS ON			@ 1S.    TOTAL 1S
;12	SINGLE W	LED 1+2+3 THEN LED 4+5+6	@0.2S.  TOTAL 4S

;USES T_MODE TO KEEP TRACK OF OF WHICH MODE AND T_TIME TO KEEP TRACK OF CURRENT TIME IN THAT MODE
TESTMODE	MOVF	T_MODE,W
	BNZ	TESTCHK
	CLRF	T_TIME   ;TEST MODE MUST HAVE BEEN ZERO SO RESET EVERYTHING

TESTCHK	;MODE WASN'T ZERO SO WORK OUT WHAT MODE IT IS IN
	BTFSC	T_MODE,3
	GOTO	TEST_GRT8		;IF BIT 3 WAS ON THEN IT'S IN MODE 8 OR HIGHER
	BTFSC	T_MODE,2
	GOTO	TEST4to7
	BTFSC	T_MODE,1
	GOTO	TEST2or3
	GOTO	TEST1
TEST2or3	BTFSS	T_MODE,0
	GOTO	TEST2
	GOTO	TEST3
TEST4to7	BTFSC	T_MODE,1
	GOTO	TEST6or7
	BTFSS	T_MODE,0
	GOTO	TEST4
	GOTO	TEST5
TEST6or7	BTFSS	T_MODE,0		;
	GOTO	TEST6
	GOTO	TEST7
TEST_GRT8	BTFSC	T_MODE,2		;CHECK TO SEE IF IN MODE 12
	GOTO	TEST12
	BTFSC	T_MODE,1		;CHECK TO SEE IF MODE 10 OR 11
	GOTO	TEST10or11
	BTFSS	T_MODE,0		;MUST BE 8 OR 9	
	GOTO	TEST8
	GOTO	TEST9
TEST10or11	BTFSS	T_MODE,0		;
	GOTO	TEST10
	GOTO	TEST11
;********************************************************** END OF THE CODE THAT CHECKS TO WHICH OF THE TEST MODES IS CURRNT ********************

TEST1	MOVLW	D'1'	;***************TEST 1	SINGLE	LED1 THEN 2 THEN 3 ----> 36 . 	@ 0.1S. TOTAL 3.6S
	MOVWF	T_MODE
	MOVF	T_TIME,W
	SUBLW	D'36'	;CHECKS TO SEE IF  ALL 36 CHANNELS HAVE BEEN TURNED ON
	BZ	TEST1_END	;if they're all done then go to the end
	CALL	SAVE_FSR
	MOVLW	RxBuffer
	ADDWF	T_TIME,W
	MOVWF	FSR0L	;UPDATE THE CHANNEL
	CLRF	FSR0H
	CLRF	INDF0	;TURN OFF OLD LED
	INCF	FSR0L,F
	MOVLW	D'255'
	MOVWF	INDF0	;TURN ON NEW LED
	INCF	T_TIME,F	;INCREMENT THE TEST TIMER
	CALL	RESTORE_FSR
	RETURN
TEST1_STT	;TEST MODE 1 START
	MOVLW	D'255'
	MOVWF	RxBuffer1	;TURN ON THE 1ST LED VIA THE DMX CHANNEL DATA
	INCF	T_TIME,F
	RETURN
TEST1_END	INCF	T_MODE,F	;END OF THE TEST 1 MODE *****************************************************
	CLRF	RxBuffer1+D'35'	;turn off led 36
	MOVLW	D'255'
	MOVWF	RxBuffer1
	CLRF	T_TIME

TEST2	;***************TEST 2	SINGLE R	LED1 THEN 4 THEN 7 ETC		@ 0.2S. TOTAL 4S
	MOVF	T_TIME,W
	BZ	TEST2_STT
	SUBLW	D'23'
	BZ	TEST2_END
	BTFSC	T_TIME,0	;CHECK THE SEE IF 0.2S
	GOTO	TEST2B	;HAS EXPIRED. IF IT HAS THEN GO TO NEXT STEP
	INCF	T_TIME,F	;ELSE INCREMENT THE TIMER AND RETURN
	RETURN
TEST2B	CALL	SAVE_FSR
	MOVF	T_TIME,W
	MOVWF	TEMP3
	RRF	TEMP3,F	;ROTATE THE T_TIME SO THAT IT IS NO LONGER DOUBLE WHAT IT SHOULD BE BECAUSE OF .2s
	BCF	TEMP3,7	;MAKE SURE MSB IS OFF
	MOVLW	RxBuffer1
	ADDWF	TEMP3,W
	ADDWF	TEMP3,W	;NEED TO ADD 3 COPIES OF THE TIME AS
	ADDWF	TEMP3,W	;IT'S EVERY 3RD LED
	MOVWF	FSR0L	;UPDATE THE CHANNEL
	CLRF	FSR0H
	CLRF	INDF0	;TURN OFF OLD LED
	INCF	FSR0L,F	;SKIP THE 
	INCF 	FSR0L,F	;NEXT 3
	INCF	FSR0L,F	;CHANNELS AND THEN
	MOVLW	D'255'
	MOVWF	INDF0	;TURN ON NEW LED
	INCF	T_TIME,F	;INCREMENT THE TEST TIMER
	CALL	RESTORE_FSR
	RETURN
TEST2_STT	INCF	T_TIME,F
	RETURN
TEST2_END	INCF	T_MODE,F	;END OF TEST 2 MODE ****************************************************
	CLRF	RxBuffer1+D'33'	;turn off the last RED led that was on
	MOVLW	D'255'
	MOVWF	RxBuffer1+D'1'	;TURN ON 1ST GREEN LED
	CLRF	T_TIME

TEST3	MOVF	T_TIME,W	;**********************TEST 3	SINGLE G	LED2 THEN 5 THEN 8 ETC		@ 0.2S. TOTAL 4S
	BZ	TEST3_STT
	SUBLW	D'23'
	BZ	TEST3_END
	BTFSC	T_TIME,0	;CHECK THE SEE IF 0.2S
	GOTO	TEST3B	;HAS EXPIRED. IF IT HAS THEN GO TO NEXT STEP
	INCF	T_TIME,F	;ELSE INCREMENT THE TIMER AND RETURN
	RETURN
TEST3B	CALL	SAVE_FSR
	MOVF	T_TIME,W
	MOVWF	TEMP3
	RRF	TEMP3,F	;ROTATE THE T_TIME SO THAT IT IS NO LONGER DOUBLE WHAT IT SHOULD BE BECAUSE OF .2s
	BCF	TEMP3,7	;MAKE SURE MSB IS OFF
	MOVLW	RxBuffer1
	ADDWF	TEMP3,W
	ADDWF	TEMP3,W	;NEED TO ADD 3 COPIES OF THE TIME AS
	ADDWF	TEMP3,W	;IT'S EVERY 3RD LED
	MOVWF	FSR0L	;UPDATE THE CHANNEL
	INCF	FSR0L,F	;INCREMENT THE CHANNEL NUMBER AS IT'S THE GREEN LEDS
	CLRF	FSR0H
	CLRF	INDF0	;TURN OFF OLD LED
	INCF	FSR0L,F	;SKIP THE 
	INCF 	FSR0L,F	;NEXT 3
	INCF	FSR0L,F	;CHANNELS AND THEN
	MOVLW	D'255'
	MOVWF	INDF0	;TURN ON NEW LED
	INCF	T_TIME,F	;INCREMENT THE TEST TIMER
	CALL	RESTORE_FSR
	RETURN
TEST3_STT	INCF	T_TIME,F
	RETURN
TEST3_END	INCF	T_MODE,F	;END OF TEST 3 MODE ****************************************************
	CLRF	RxBuffer1+D'34'	;turn off the last green led that was on
	MOVLW	D'255'
	MOVWF	RxBuffer1+D'2'	;TURN ON THE 1ST BLUE LED
	CLRF	T_TIME

TEST4	;**********************TEST 4	SINGLE B	LED3 THEN 6 THEN 9 ETC		@ 0.2S. TOTAL 4S
	MOVF	T_TIME,W
	BZ	TEST4_STT	;JUMP TO THE START IF IT'S JUST ENTERING THE MODE
	SUBLW	D'23'
	BZ	TEST4_END
	BTFSC	T_TIME,0	;CHECK THE SEE IF 0.2S
	GOTO	TEST4B	;HAS EXPIRED. IF IT HAS THEN GO TO NEXT STEP
	INCF	T_TIME,F	;ELSE INCREMENT THE TIMER AND RETURN
	RETURN
TEST4B	CALL	SAVE_FSR
	MOVF	T_TIME,W
	MOVWF	TEMP3
	RRF	TEMP3,F	;ROTATE THE T_TIME SO THAT IT IS NO LONGER DOUBLE WHAT IT SHOULD BE BECAUSE OF .2s
	BCF	TEMP3,7	;MAKE SURE MSB IS OFF
	MOVLW	RxBuffer1
	ADDWF	TEMP3,W
	ADDWF	TEMP3,W	;NEED TO ADD 3 COPIES OF THE TIME AS
	ADDWF	TEMP3,W	;IT'S EVERY 3RD LED
	MOVWF	FSR0L	;UPDATE THE CHANNEL
	INCF	FSR0L,F	;INCREMENT THE CHANNEL NUMBER AS IT'S THE BLUE LEDS
	INCF	FSR0L,F	;INCREMENT THE CHANNEL NUMBER AS IT'S THE BLUE LEDS
	CLRF	FSR0H
	CLRF	INDF0	;TURN OFF OLD LED
	INCF	FSR0L,F	;SKIP THE 
	INCF 	FSR0L,F	;NEXT 3
	INCF	FSR0L,F	;CHANNELS AND THEN
	MOVLW	D'255'
	MOVWF	INDF0	;TURN ON NEW LED
	INCF	T_TIME,F	;INCREMENT THE TEST TIMER
	CALL	RESTORE_FSR
	RETURN
TEST4_STT	MOVLW	D'255'
	MOVWF	RxBuffer1+D'2'	;TURN ON 1ST BLUE LED
	INCF	T_TIME,F
	RETURN
TEST4_END	INCF	T_MODE,F	;END OF TEST 4 MODE ****************************************************
	CLRF	RxBuffer1+D'35' ;turn off the last BLUE led that was on
	CLRF	T_TIME

TEST5	;**********************TEST 5	ALL RED	LED 1 + LED 4 + LED 7 ETC		@ 1S.    TOTAL 1S
	MOVF	T_TIME,W
	SUBLW	D'11'
	BZ	TEST5_END
	MOVLW	D'255'
	MOVWF	RxBuffer1	;RED
	MOVWF	RxBuffer1+D'3'
	MOVWF	RxBuffer1+D'6'
	MOVWF	RxBuffer1+D'9'
	MOVWF	RxBuffer1+D'12'
	MOVWF	RxBuffer1+D'15'
	MOVWF	RxBuffer1+D'18'
	MOVWF	RxBuffer1+D'21'
	MOVWF	RxBuffer1+D'24'
	MOVWF	RxBuffer1+D'27'
	MOVWF	RxBuffer1+D'30'
	MOVWF	RxBuffer1+D'33'
	INCF	T_TIME,F
	RETURN
TEST5_END	INCF	T_MODE,F
	CLRF	RxBuffer1	;RED
	CLRF	RxBuffer1+D'3'
	CLRF	RxBuffer1+D'6'
	CLRF	RxBuffer1+D'9'
	CLRF	RxBuffer1+D'12'
	CLRF	RxBuffer1+D'15'
	CLRF	RxBuffer1+D'18'
	CLRF	RxBuffer1+D'21'
	CLRF	RxBuffer1+D'24'
	CLRF	RxBuffer1+D'27'
	CLRF	RxBuffer1+D'30'
	CLRF	RxBuffer1+D'33'
	CLRF	T_TIME

TEST6	MOVF	T_TIME,W		;**********************TEST 6	ALL GREEN	LED 2 + LED 5 + LED 8 ETC		@ 1S.    TOTAL 1S
	SUBLW	D'11'
	BZ	TEST6_END
	MOVLW	D'255'
	MOVWF	RxBuffer1+D'1'	;GREEN
	MOVWF	RxBuffer1+D'4'
	MOVWF	RxBuffer1+D'7'
	MOVWF	RxBuffer1+D'10'
	MOVWF	RxBuffer1+D'13'
	MOVWF	RxBuffer1+D'16'
	MOVWF	RxBuffer1+D'19'
	MOVWF	RxBuffer1+D'22'
	MOVWF	RxBuffer1+D'25'
	MOVWF	RxBuffer1+D'28'
	MOVWF	RxBuffer1+D'31'
	MOVWF	RxBuffer1+D'34'
	INCF	T_TIME,F
	RETURN

TEST6_END	INCF	T_MODE,F
	CLRF	RxBuffer1+D'1'	;green
	CLRF	RxBuffer1+D'4'
	CLRF	RxBuffer1+D'7'
	CLRF	RxBuffer1+D'10'
	CLRF	RxBuffer1+D'13'
	CLRF	RxBuffer1+D'16'
	CLRF	RxBuffer1+D'19'
	CLRF	RxBuffer1+D'22'
	CLRF	RxBuffer1+D'25'
	CLRF	RxBuffer1+D'28'
	CLRF	RxBuffer1+D'31'
	CLRF	RxBuffer1+D'34'
	CLRF	T_TIME

TEST7	MOVF	T_TIME,W	;**********************TEST 7	ALL BLUE	LED 3 + LED 6 + LED 9 ETC		@ 1S.    TOTAL 1S
	SUBLW	D'11'
	BZ	TEST7_END
	;*********************TEST 7 MODE CODE GOES HERE
	MOVLW	D'255'
	MOVWF	RxBuffer1+D'2'  	;BLUE
	MOVWF	RxBuffer1+D'5'
	MOVWF	RxBuffer1+D'8'
	MOVWF	RxBuffer1+D'11'
	MOVWF	RxBuffer1+D'14'
	MOVWF	RxBuffer1+D'17'
	MOVWF	RxBuffer1+D'20'
	MOVWF	RxBuffer1+D'23'
	MOVWF	RxBuffer1+D'26'
	MOVWF	RxBuffer1+D'29'
	MOVWF	RxBuffer1+D'32'
	MOVWF	RxBuffer1+D'35'
	INCF	T_TIME,F
	RETURN
TEST7_END	INCF	T_MODE,F		;blue leds get left on for the next mode
	CLRF	T_TIME

TEST8	;**********************TEST 8	AQUA (G+B) 	LED 2 +LED 3 + LED 5 + LED 6 ETC	@ 1S.    TOTAL 1S
	MOVF	T_TIME,W
	SUBLW	D'11'
	BZ	TEST8_END
	MOVLW	D'255'
	MOVWF	RxBuffer1+D'1'	;GREEN
	MOVWF	RxBuffer1+D'4'
	MOVWF	RxBuffer1+D'7'
	MOVWF	RxBuffer1+D'10'
	MOVWF	RxBuffer1+D'13'
	MOVWF	RxBuffer1+D'16'
	MOVWF	RxBuffer1+D'19'
	MOVWF	RxBuffer1+D'22'
	MOVWF	RxBuffer1+D'25'
	MOVWF	RxBuffer1+D'28'
	MOVWF	RxBuffer1+D'31'
	MOVWF	RxBuffer1+D'34'
	INCF	T_TIME,F
	RETURN
TEST8_END	INCF	T_MODE,F
	CLRF	RxBuffer1+D'2'	;BLUE  	;BLUES GET TURNED OFF, GREENS STAY ON
	CLRF	RxBuffer1+D'5'
	CLRF	RxBuffer1+D'8'
	CLRF	RxBuffer1+D'11'
	CLRF	RxBuffer1+D'14'
	CLRF	RxBuffer1+D'17'
	CLRF	RxBuffer1+D'20'
	CLRF	RxBuffer1+D'23'
	CLRF	RxBuffer1+D'26'
	CLRF	RxBuffer1+D'29'
	CLRF	RxBuffer1+D'32'
	CLRF	RxBuffer1+D'35'
	CLRF	T_TIME

TEST9	;**********************TEST 9	YELLOW (R+G)	LED 1 + LED 2 + LED 4 + LED 5 ETC	@ 1S.    TOTAL 1S
	MOVF	T_TIME,W
	SUBLW	D'11'
	BZ	TEST9_END
	MOVLW	D'255'
	MOVWF	RxBuffer1	;RED
	MOVWF	RxBuffer1+D'3'
	MOVWF	RxBuffer1+D'6'
	MOVWF	RxBuffer1+D'9'
	MOVWF	RxBuffer1+D'12'
	MOVWF	RxBuffer1+D'15'
	MOVWF	RxBuffer1+D'18'
	MOVWF	RxBuffer1+D'21'
	MOVWF	RxBuffer1+D'24'
	MOVWF	RxBuffer1+D'27'
	MOVWF	RxBuffer1+D'30'
	MOVWF	RxBuffer1+D'33'
	INCF	T_TIME,F
	RETURN
TEST9_END	INCF	T_MODE,F
	CLRF	RxBuffer1+D'1'	;green	;TURN OFF THE GREEN, RED STAYS ON AS THEY ARE NEEDED
	CLRF	RxBuffer1+D'4'
	CLRF	RxBuffer1+D'7'
	CLRF	RxBuffer1+D'10'
	CLRF	RxBuffer1+D'13'
	CLRF	RxBuffer1+D'16'
	CLRF	RxBuffer1+D'19'
	CLRF	RxBuffer1+D'22'
	CLRF	RxBuffer1+D'25'
	CLRF	RxBuffer1+D'28'
	CLRF	RxBuffer1+D'31'
	CLRF	RxBuffer1+D'34'
	CLRF	T_TIME

TEST10	;**********************TEST 10	PURPLE (R+B)	LED 1 + LED 3 + LED 4 + LED 6 ETC	@ 1S.    TOTAL 1S
	MOVF	T_TIME,W
	SUBLW	D'11'
	BZ	TEST10_END
	;*********************TEST 10 MODE CODE GOES HERE
	MOVLW	D'255'
	MOVWF	RxBuffer1+D'2'  	;BLUE
	MOVWF	RxBuffer1+D'5'
	MOVWF	RxBuffer1+D'8'
	MOVWF	RxBuffer1+D'11'
	MOVWF	RxBuffer1+D'14'
	MOVWF	RxBuffer1+D'17'
	MOVWF	RxBuffer1+D'20'
	MOVWF	RxBuffer1+D'23'
	MOVWF	RxBuffer1+D'26'
	MOVWF	RxBuffer1+D'29'
	MOVWF	RxBuffer1+D'32'
	MOVWF	RxBuffer1+D'35'
	INCF	T_TIME,F
	RETURN

TEST10_END	INCF	T_MODE,F	;NO LEDS NEED TO BE TURNED OFF
	CLRF	T_TIME

TEST11	MOVF	T_TIME,W	;**********************TEST 11	WHITE	ALL LEDS ON			@ 1S.    TOTAL 1S
	SUBLW	D'11'
	BZ	TEST11_END
	MOVLW	D'255'
	MOVWF	RxBuffer1+D'1'	;GREEN	REDS AND BLUES WERE ALREADY ON
	MOVWF	RxBuffer1+D'4'
	MOVWF	RxBuffer1+D'7'
	MOVWF	RxBuffer1+D'10'
	MOVWF	RxBuffer1+D'13'
	MOVWF	RxBuffer1+D'16'
	MOVWF	RxBuffer1+D'19'
	MOVWF	RxBuffer1+D'22'
	MOVWF	RxBuffer1+D'25'
	MOVWF	RxBuffer1+D'28'
	MOVWF	RxBuffer1+D'31'
	MOVWF	RxBuffer1+D'34'
	INCF	T_TIME,F
	RETURN
TEST11_END	INCF	T_MODE,F
	CALL	AllOff	;CLEAR ALL THE LEDS
	CLRF	T_TIME

TEST12	MOVF	T_TIME,W	;**********************TEST 12	SINGLE W	LED 1+2+3 THEN LED 4+5+6	@0.2S.  TOTAL 4S
	BZ	TEST12_STT
;*************
	SUBLW	D'39'
	BZ	TEST12_END
	BTFSC	T_TIME,0	;CHECK THE SEE IF 0.2S
	GOTO	TEST12B	;HAS EXPIRED. IF IT HAS THEN GO TO NEXT STEP
	INCF	T_TIME,F	;ELSE INCREMENT THE TIMER AND RETURN
	RETURN
	
TEST12B	CALL	SAVE_FSR
	MOVF	T_TIME,W
	MOVWF	TEMP3
	RRF	TEMP3,F	;ROTATE THE T_TIME SO THAT IT IS NO LONGER DOUBLE WHAT IT SHOULD BE BECAUSE OF .2s
	BCF	TEMP3,7	;MAKE SURE MSB IS OFF
	MOVLW	RxBuffer1
	ADDWF	TEMP3,W
	ADDWF	TEMP3,W	;NEED TO ADD 3 COPIES OF THE TIME AS
	ADDWF	TEMP3,W	;IT'S EVERY 3RD LED
	MOVWF	FSR0L	;UPDATE THE CHANNEL
 	CLRF	FSR0H

	CLRF	FSR0	;TURN OFF OLD LED
	INCF	FSR0L,F
	CLRF	INDF0	;TURN OFF OLD LED
	INCF	FSR0L,F
	CLRF	INDF0	;TURN OFF OLD LED
	INCF	FSR0L,F
	MOVLW	D'255'
	MOVWF	INDF0	;TURN ON NEW LED
	INCF	FSR0L,F
	MOVWF	INDF0	;TURN ON NEW LED
	INCF	FSR0L,F
	MOVWF	INDF0	;TURN ON NEW LED
	INCF	FSR0L,F
	CALL	RESTORE_FSR
	INCF	T_TIME,F
	RETURN
TEST12_STT	MOVLW	D'255'
	MOVWF	RxBuffer1	;TURN ON 1ST RED LED
	MOVWF	RxBuffer1+D'1'	;TURN ON 1ST GREEN LED
	MOVWF	RxBuffer1+D'2'	;TURN ON 1ST BLUE LED
	INCF	T_TIME,F
	RETURN
TEST12_END	CLRF	T_MODE	;RESET THE MODE
	CALL	AllOff	;RESET ALL THE LEDS
	CLRF	T_TIME	;RESET THE TIMER
	RETURN

AllOff	CLRF	RxBuffer1	;RED	CLEAR ALL 36 CHANNELS
	CLRF	RxBuffer1+D'3'
	CLRF	RxBuffer1+D'6'
	CLRF	RxBuffer1+D'9'
	CLRF	RxBuffer1+D'12'
	CLRF	RxBuffer1+D'15'
	CLRF	RxBuffer1+D'18'
	CLRF	RxBuffer1+D'21'
	CLRF	RxBuffer1+D'24'
	CLRF	RxBuffer1+D'27'
	CLRF	RxBuffer1+D'30'
	CLRF	RxBuffer1+D'33'
	CLRF	RxBuffer1+D'1'	;green
	CLRF	RxBuffer1+D'4'
	CLRF	RxBuffer1+D'7'
	CLRF	RxBuffer1+D'10'
	CLRF	RxBuffer1+D'13'
	CLRF	RxBuffer1+D'16'
	CLRF	RxBuffer1+D'19'
	CLRF	RxBuffer1+D'22'
	CLRF	RxBuffer1+D'25'
	CLRF	RxBuffer1+D'28'
	CLRF	RxBuffer1+D'31'
	CLRF	RxBuffer1+D'34'
	CLRF	RxBuffer1+D'2'	;BLUE
	CLRF	RxBuffer1+D'5'
	CLRF	RxBuffer1+D'8'
	CLRF	RxBuffer1+D'11'
	CLRF	RxBuffer1+D'14'
	CLRF	RxBuffer1+D'17'
	CLRF	RxBuffer1+D'20'
	CLRF	RxBuffer1+D'23'
	CLRF	RxBuffer1+D'26'
	CLRF	RxBuffer1+D'29'
	CLRF	RxBuffer1+D'32'
	CLRF	RxBuffer1+D'35'
	RETURN
;****************************************************************************** END OF TEST MODE **********************************************
	END
